@@ -8,19 +8,19 @@ use std::fmt::Write;
 use std::time::SystemTime;
 
 use eden_dag::DagAlgorithm;
+use rayon::ThreadPoolBuilder;
 use tracing::instrument;
 
 use crate::core::config::get_restack_preserve_timestamps;
+use crate::core::dag::{resolve_commits, CommitSet, Dag, ResolveCommitsResult};
+use crate::core::effects::Effects;
 use crate::core::eventlog::{EventLogDb, EventReplayer};
 use crate::core::rewrite::{
     execute_rebase_plan, BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
-    RebasePlanBuilder,
-};
-use crate::git::{
-    resolve_commits, CommitSet, Dag, GitRunInfo, NonZeroOid, Repo, ResolveCommitsResult,
+    RebasePlanBuilder, RepoResource,
 };
+use crate::git::{GitRunInfo, NonZeroOid, Repo};
 use crate::opts::MoveOptions;
-use crate::tui::Effects;
 
 #[instrument]
 fn resolve_base_commit(
@@ -129,21 +129,26 @@ pub fn r#move(
     let MoveOptions {
         force_in_memory,
         force_on_disk,
+        detect_duplicate_commits_via_patch_id,
         resolve_merge_conflicts,
         dump_rebase_constraints,
         dump_rebase_plan,
     } = *move_options;
     let now = SystemTime::now();
     let event_tx_id = event_log_db.make_transaction_id(now, "move")?;
+    let pool = ThreadPoolBuilder::new().build()?;
+    let repo_pool = RepoResource::new_pool(&repo)?;
     let rebase_plan = {
-        let mut builder = RebasePlanBuilder::new(&repo, &dag);
+        let mut builder = RebasePlanBuilder::new(&dag);
         builder.move_subtree(source_oid, dest_oid)?;
         builder.build(
             effects,
+            &pool,
+            &repo_pool,
             &BuildRebasePlanOptions {
                 dump_rebase_constraints,
                 dump_rebase_plan,
-                detect_duplicate_commits_via_patch_id: true,
+                detect_duplicate_commits_via_patch_id,
             },
         )?
     };
@@ -160,6 +165,7 @@ pub fn r#move(
                 force_in_memory,
                 force_on_disk,
                 resolve_merge_conflicts,
+                check_out_commit_options: Default::default(),
             };
             execute_rebase_plan(effects, git_run_info, &repo, &rebase_plan, &options)?
         }
@@ -170,7 +176,7 @@ pub fn r#move(
     };
 
     match result {
-        ExecuteRebasePlanResult::Succeeded => Ok(0),
+        ExecuteRebasePlanResult::Succeeded { rewritten_oids: _ } => Ok(0),
 
         ExecuteRebasePlanResult::DeclinedToMerge { merge_conflict } => {
             merge_conflict.describe(effects, &repo)?;
