@@ -19,9 +19,9 @@ use crate::core::eventlog::{EventLogDb, EventReplayer};
 use crate::core::formatting::{printable_styled_string, Pluralize};
 use crate::core::node_descriptors::{
     BranchesDescriptor, CommitMessageDescriptor, CommitOidDescriptor,
-    DifferentialRevisionDescriptor, NodeDescriptor, RelativeTimeDescriptor,
+    DifferentialRevisionDescriptor, NodeDescriptor, Redactor, RelativeTimeDescriptor,
 };
-use crate::git::{check_out_commit, GitRunInfo, NonZeroOid, Repo};
+use crate::git::{check_out_commit, CheckOutCommitOptions, GitRunInfo, NonZeroOid, Repo};
 use crate::opts::{CheckoutOptions, TraverseCommitsOptions};
 use crate::tui::prompt_select_commit;
 
@@ -207,21 +207,20 @@ fn advance(
 
         let pluralize = match command {
             Command::Next => Pluralize {
+                determiner: None,
                 amount: i.try_into()?,
-                plural: "children",
-                singular: "child",
+                unit: ("child", "children"),
             },
 
             Command::Prev => Pluralize {
+                determiner: None,
                 amount: i.try_into()?,
-                plural: "parents",
-                singular: "parent",
+                unit: ("parent", "parents"),
             },
         };
         let header = format!(
             "Found multiple possible {} commits to go to after traversing {}:",
-            pluralize.singular,
-            pluralize.to_string(),
+            pluralize.unit.0, pluralize,
         );
 
         current_oid = match (towards, candidate_commits.as_slice()) {
@@ -234,8 +233,7 @@ fn advance(
                         StyledString::styled(
                             format!(
                                 "No more {} commits to go to after traversing {}.",
-                                pluralize.singular,
-                                pluralize.to_string(),
+                                pluralize.unit.0, pluralize,
                             ),
                             BaseColor::Yellow.light()
                         )
@@ -283,7 +281,7 @@ fn advance(
                         effects.get_output_stream(),
                         "  {} {}{}",
                         glyphs.bullet_point,
-                        printable_styled_string(glyphs, child.friendly_describe()?)?,
+                        printable_styled_string(glyphs, child.friendly_describe(glyphs)?)?,
                         descriptor
                     )?;
                 }
@@ -345,6 +343,7 @@ pub fn traverse_commits(
     };
 
     let repo = Repo::from_current_dir()?;
+    let head_info = repo.get_head_info()?;
     let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let event_log_db = EventLogDb::new(&conn)?;
@@ -372,9 +371,14 @@ pub fn traverse_commits(
         &mut [
             &mut CommitOidDescriptor::new(true)?,
             &mut RelativeTimeDescriptor::new(&repo, SystemTime::now())?,
-            &mut BranchesDescriptor::new(&repo, &references_snapshot)?,
-            &mut DifferentialRevisionDescriptor::new(&repo)?,
-            &mut CommitMessageDescriptor::new()?,
+            &mut BranchesDescriptor::new(
+                &repo,
+                &head_info,
+                &references_snapshot,
+                &Redactor::Disabled,
+            )?,
+            &mut DifferentialRevisionDescriptor::new(&repo, &Redactor::Disabled)?,
+            &mut CommitMessageDescriptor::new(&Redactor::Disabled)?,
         ],
         head_oid,
         command,
@@ -427,7 +431,7 @@ pub fn traverse_commits(
             args.push("--merge");
         }
         if force {
-            args.push("--force");
+            args.push("--force")
         }
         args
     };
@@ -436,7 +440,10 @@ pub fn traverse_commits(
         git_run_info,
         None,
         Some(&current_oid),
-        additional_args.as_slice(),
+        &CheckOutCommitOptions {
+            additional_args: additional_args.as_slice(),
+            ..Default::default()
+        },
     )
 }
 
@@ -519,6 +526,7 @@ pub fn checkout(
     } = checkout_options;
 
     let repo = Repo::from_current_dir()?;
+    let head_info = repo.get_head_info()?;
     let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let event_log_db = EventLogDb::new(&conn)?;
@@ -532,7 +540,15 @@ pub fn checkout(
         &references_snapshot,
     )?;
 
-    let graph = make_smartlog_graph(effects, &repo, &dag, &event_replayer, event_cursor, true)?;
+    let graph = make_smartlog_graph(
+        effects,
+        &repo,
+        &dag,
+        &event_replayer,
+        event_cursor,
+        true,
+        false,
+    )?;
 
     let initial_query = get_initial_query(checkout_options);
     let target = match initial_query {
@@ -545,9 +561,14 @@ pub fn checkout(
                 &mut [
                     &mut CommitOidDescriptor::new(true)?,
                     &mut RelativeTimeDescriptor::new(&repo, SystemTime::now())?,
-                    &mut BranchesDescriptor::new(&repo, &references_snapshot)?,
-                    &mut DifferentialRevisionDescriptor::new(&repo)?,
-                    &mut CommitMessageDescriptor::new()?,
+                    &mut BranchesDescriptor::new(
+                        &repo,
+                        &head_info,
+                        &references_snapshot,
+                        &Redactor::Disabled,
+                    )?,
+                    &mut DifferentialRevisionDescriptor::new(&repo, &Redactor::Disabled)?,
+                    &mut CommitMessageDescriptor::new(&Redactor::Disabled)?,
                 ],
             )? {
                 Some(oid) => Some(oid.to_string()),
@@ -576,7 +597,10 @@ pub fn checkout(
         git_run_info,
         None,
         target.as_ref(),
-        additional_args,
+        &CheckOutCommitOptions {
+            additional_args: additional_args.as_slice(),
+            render_smartlog: true,
+        },
     )?;
     Ok(exit_code)
 }
