@@ -14,7 +14,8 @@ use crate::core::eventlog::{EventLogDb, EventReplayer};
 use crate::core::formatting::printable_styled_string;
 use crate::core::node_descriptors::{
     BranchesDescriptor, CommitMessageDescriptor, CommitOidDescriptor,
-    DifferentialRevisionDescriptor, ObsolescenceExplanationDescriptor, RelativeTimeDescriptor,
+    DifferentialRevisionDescriptor, ObsolescenceExplanationDescriptor, Redactor,
+    RelativeTimeDescriptor,
 };
 use crate::git::{GitRunInfo, Repo};
 
@@ -223,6 +224,7 @@ mod graph {
         event_replayer: &EventReplayer,
         event_cursor: EventCursor,
         remove_commits: bool,
+        only_branches: bool,
     ) -> eyre::Result<SmartlogGraph<'repo>> {
         let (effects, _progress) = effects.start_operation(OperationType::MakeGraph);
 
@@ -230,15 +232,17 @@ mod graph {
             let (effects, _progress) = effects.start_operation(OperationType::WalkCommits);
 
             let public_commits = dag.query_public_commits()?;
-            let active_heads = if remove_commits {
-                dag.query_active_heads(
-                    &public_commits,
-                    &dag.observed_commits.difference(&dag.obsolete_commits),
-                )?
+
+            let observed_commits = if only_branches {
+                dag.branch_commits.clone()
+            } else if remove_commits {
+                dag.observed_commits.difference(&dag.obsolete_commits)
             } else {
-                dag.query_active_heads(&public_commits, &dag.observed_commits.clone())?
+                dag.observed_commits.clone()
             };
 
+            let active_heads = dag.query_active_heads(&public_commits, &observed_commits)?;
+
             walk_from_active_heads(
                 &effects,
                 repo,
@@ -338,7 +342,7 @@ mod render {
         let current_node = &graph[&current_oid];
         let is_head = Some(current_oid) == head_oid;
 
-        let text = render_node_descriptors(&current_node.object, commit_descriptors)?;
+        let text = render_node_descriptors(glyphs, &current_node.object, commit_descriptors)?;
         let cursor = match (current_node.is_main, current_node.is_obsolete, is_head) {
             (false, false, false) => glyphs.commit_visible,
             (false, false, true) => glyphs.commit_visible_head,
@@ -519,6 +523,9 @@ mod render {
         /// Whether to also show commits in the smartlog which would normally not be
         /// visible.
         pub show_hidden_commits: bool,
+
+        /// Whether to only show commits on branches.
+        pub only_show_branches: bool,
     }
 }
 
@@ -531,9 +538,11 @@ pub fn smartlog(
 ) -> eyre::Result<()> {
     let SmartlogOptions {
         show_hidden_commits,
+        only_show_branches,
     } = options;
 
     let repo = Repo::from_dir(&git_run_info.working_directory)?;
+    let head_info = repo.get_head_info()?;
     let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let event_log_db = EventLogDb::new(&conn)?;
@@ -554,6 +563,7 @@ pub fn smartlog(
         &event_replayer,
         event_cursor,
         !show_hidden_commits,
+        *only_show_branches,
     )?;
 
     let lines = render_graph(
@@ -569,9 +579,14 @@ pub fn smartlog(
                 &event_replayer,
                 event_replayer.make_default_cursor(),
             )?,
-            &mut BranchesDescriptor::new(&repo, &references_snapshot)?,
-            &mut DifferentialRevisionDescriptor::new(&repo)?,
-            &mut CommitMessageDescriptor::new()?,
+            &mut BranchesDescriptor::new(
+                &repo,
+                &head_info,
+                &references_snapshot,
+                &Redactor::Disabled,
+            )?,
+            &mut DifferentialRevisionDescriptor::new(&repo, &Redactor::Disabled)?,
+            &mut CommitMessageDescriptor::new(&Redactor::Disabled)?,
         ],
     )?;
     for line in lines {
