@@ -11,7 +11,7 @@ use std::sync::mpsc::{channel, Receiver, Sender, TryRecvError};
 use std::time::SystemTime;
 
 use cursive::event::Key;
-use cursive::traits::Boxable;
+use cursive::traits::Resizable;
 use cursive::utils::markup::StyledString;
 use cursive::views::{Dialog, EditView, LinearLayout, OnEventView, Panel, ScrollView, TextView};
 use cursive::{Cursive, CursiveRunnable, CursiveRunner};
@@ -22,13 +22,17 @@ use crate::commands::smartlog::{make_smartlog_graph, render_graph};
 use crate::core::dag::Dag;
 use crate::core::effects::Effects;
 use crate::core::eventlog::{Event, EventCursor, EventLogDb, EventReplayer, EventTransactionId};
-use crate::core::formatting::{printable_styled_string, Pluralize, StyledStringBuilder};
+use crate::core::formatting::{printable_styled_string, Glyphs, Pluralize, StyledStringBuilder};
 use crate::core::node_descriptors::{
     BranchesDescriptor, CommitMessageDescriptor, CommitOidDescriptor,
-    DifferentialRevisionDescriptor, ObsolescenceExplanationDescriptor, RelativeTimeDescriptor,
+    DifferentialRevisionDescriptor, ObsolescenceExplanationDescriptor, Redactor,
+    RelativeTimeDescriptor,
 };
 use crate::declare_views;
-use crate::git::{check_out_commit, CategorizedReferenceName, GitRunInfo, MaybeZeroOid, Repo};
+use crate::git::{
+    check_out_commit, CategorizedReferenceName, CheckOutCommitOptions, GitRunInfo, MaybeZeroOid,
+    Repo, ResolvedReferenceInfo,
+};
 use crate::tui::{with_siv, SingletonView};
 
 fn render_cursor_smartlog(
@@ -40,7 +44,24 @@ fn render_cursor_smartlog(
 ) -> eyre::Result<Vec<StyledString>> {
     let dag = dag.set_cursor(effects, repo, event_replayer, event_cursor)?;
     let references_snapshot = event_replayer.get_references_snapshot(repo, event_cursor)?;
-    let graph = make_smartlog_graph(effects, repo, &dag, event_replayer, event_cursor, true)?;
+
+    // FIXME: the information of which branch is checked out is not stored in
+    // the event log. For now, just pretend that `HEAD` was always detached in
+    // the past.
+    let head_info = ResolvedReferenceInfo {
+        oid: references_snapshot.head_oid,
+        reference_name: None,
+    };
+
+    let graph = make_smartlog_graph(
+        effects,
+        repo,
+        &dag,
+        event_replayer,
+        event_cursor,
+        true,
+        false,
+    )?;
     let result = render_graph(
         effects,
         repo,
@@ -51,15 +72,20 @@ fn render_cursor_smartlog(
             &mut CommitOidDescriptor::new(true)?,
             &mut RelativeTimeDescriptor::new(repo, SystemTime::now())?,
             &mut ObsolescenceExplanationDescriptor::new(event_replayer, event_cursor)?,
-            &mut BranchesDescriptor::new(repo, &references_snapshot)?,
-            &mut DifferentialRevisionDescriptor::new(repo)?,
-            &mut CommitMessageDescriptor::new()?,
+            &mut BranchesDescriptor::new(
+                repo,
+                &head_info,
+                &references_snapshot,
+                &Redactor::Disabled,
+            )?,
+            &mut DifferentialRevisionDescriptor::new(repo, &Redactor::Disabled)?,
+            &mut CommitMessageDescriptor::new(&Redactor::Disabled)?,
         ],
     )?;
     Ok(result)
 }
 
-fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>> {
+fn describe_event(glyphs: &Glyphs, repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>> {
     // Links to https://github.com/arxanas/git-branchless/issues/57
     const EMPTY_EVENT_MESSAGE: &str =
         "This may be an unsupported use-case; see https://git.io/J0b7z";
@@ -73,7 +99,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Commit ")
-                    .append(repo.friendly_describe_commit_from_oid(*commit_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *commit_oid)?)
                     .build(),
                 StyledString::new(),
             ]
@@ -93,7 +119,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Hide commit ")
-                    .append(repo.friendly_describe_commit_from_oid(*commit_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *commit_oid)?)
                     .build(),
                 StyledString::new(),
             ]
@@ -113,7 +139,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Unhide commit ")
-                    .append(repo.friendly_describe_commit_from_oid(*commit_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *commit_oid)?)
                     .build(),
                 StyledString::new(),
             ]
@@ -131,7 +157,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Check out to ")
-                    .append(repo.friendly_describe_commit_from_oid(*new_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *new_oid)?)
                     .build(),
                 StyledString::new(),
             ]
@@ -148,11 +174,11 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Check out from ")
-                    .append(repo.friendly_describe_commit_from_oid(*old_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *old_oid)?)
                     .build(),
                 StyledStringBuilder::new()
                     .append_plain("            to ")
-                    .append(repo.friendly_describe_commit_from_oid(*new_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *new_oid)?)
                     .build(),
             ]
         }
@@ -189,7 +215,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
                     .append_plain("Create ")
                     .append_plain(CategorizedReferenceName::new(ref_name).friendly_describe())
                     .append_plain(" at ")
-                    .append(repo.friendly_describe_commit_from_oid(*new_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *new_oid)?)
                     .build(),
                 StyledString::new(),
             ]
@@ -208,7 +234,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
                     .append_plain("Delete ")
                     .append_plain(CategorizedReferenceName::new(ref_name).friendly_describe())
                     .append_plain(" at ")
-                    .append(repo.friendly_describe_commit_from_oid(*old_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *old_oid)?)
                     .build(),
                 StyledString::new(),
             ]
@@ -228,13 +254,13 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
                     .append_plain("Move ")
                     .append_plain(ref_name.clone())
                     .append_plain(" from ")
-                    .append(repo.friendly_describe_commit_from_oid(*old_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *old_oid)?)
                     .build(),
                 StyledStringBuilder::new()
                     .append_plain("     ")
                     .append_plain(" ".repeat(ref_name.len()))
                     .append_plain("   to ")
-                    .append(repo.friendly_describe_commit_from_oid(*new_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *new_oid)?)
                     .build(),
             ]
         }
@@ -248,11 +274,11 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Rewrite commit ")
-                    .append(repo.friendly_describe_commit_from_oid(*old_commit_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *old_commit_oid)?)
                     .build(),
                 StyledStringBuilder::new()
                     .append_plain("           as ")
-                    .append(repo.friendly_describe_commit_from_oid(*new_commit_oid)?)
+                    .append(repo.friendly_describe_commit_from_oid(glyphs, *new_commit_oid)?)
                     .build(),
             ]
         }
@@ -277,13 +303,14 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
 }
 
 fn describe_events_numbered(
+    glyphs: &Glyphs,
     repo: &Repo,
     events: &[Event],
 ) -> Result<Vec<StyledString>, eyre::Error> {
     let mut lines = Vec::new();
     for (i, event) in (1..).zip(events) {
         let num_header = format!("{}. ", i);
-        for (j, event_line) in (0..).zip(describe_event(repo, event)?) {
+        for (j, event_line) in (0..).zip(describe_event(glyphs, repo, event)?) {
             let prefix = if j == 0 {
                 num_header.clone()
             } else {
@@ -383,7 +410,8 @@ fn select_past_event(
                     "There are no previous available events.",
                 )],
                 Some((event_id, events)) => {
-                    let event_description_lines = describe_events_numbered(repo, events)?;
+                    let event_description_lines =
+                        describe_events_numbered(effects.get_glyphs(), repo, events)?;
                     let relative_time_provider = RelativeTimeDescriptor::new(repo, now)?;
                     let relative_time = if relative_time_provider.is_enabled() {
                         format!(
@@ -654,7 +682,7 @@ fn undo_events(
     }
 
     writeln!(effects.get_output_stream(), "Will apply these actions:")?;
-    let events = describe_events_numbered(repo, &inverse_events)?;
+    let events = describe_events_numbered(effects.get_glyphs(), repo, &inverse_events)?;
     for line in events {
         writeln!(
             effects.get_output_stream(),
@@ -681,9 +709,9 @@ fn undo_events(
     }
 
     let num_inverse_events = Pluralize {
+        determiner: None,
         amount: inverse_events.len().try_into().unwrap(),
-        singular: "inverse event",
-        plural: "inverse events",
+        unit: ("inverse event", "inverse events"),
     }
     .to_string();
 
@@ -708,7 +736,10 @@ fn undo_events(
                     git_run_info,
                     Some(event_tx_id),
                     Some(target_oid.as_os_str()),
-                    &["--detach"],
+                    &CheckOutCommitOptions {
+                        additional_args: &["--detach"],
+                        render_smartlog: true,
+                    },
                 )
                 .wrap_err("Updating to previous HEAD location")?;
                 if exit_code != 0 {
@@ -782,7 +813,11 @@ fn undo_events(
 
 /// Restore the repository to a previous state interactively.
 #[instrument]
-pub fn undo(effects: &Effects, git_run_info: &GitRunInfo) -> eyre::Result<isize> {
+pub fn undo(
+    effects: &Effects,
+    git_run_info: &GitRunInfo,
+    interactive: bool,
+) -> eyre::Result<isize> {
     let repo = Repo::from_current_dir()?;
     let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
@@ -803,12 +838,16 @@ pub fn undo(effects: &Effects, git_run_info: &GitRunInfo) -> eyre::Result<isize>
     };
 
     let event_cursor = {
-        let result = with_siv(effects, |effects, siv| {
-            select_past_event(siv, &effects, &repo, &dag, &mut event_replayer)
-        })?;
-        match result {
-            Some(event_cursor) => event_cursor,
-            None => return Ok(0),
+        if interactive {
+            let result = with_siv(effects, |effects, siv| {
+                select_past_event(siv, &effects, &repo, &dag, &mut event_replayer)
+            })?;
+            match result {
+                Some(event_cursor) => event_cursor,
+                None => return Ok(0),
+            }
+        } else {
+            event_replayer.advance_cursor_by_transaction(event_replayer.make_default_cursor(), -1)
         }
     };
 
