@@ -1,6 +1,5 @@
 //! Wrappers around various side effects.
 
-use std::collections::HashMap;
 use std::convert::TryInto;
 use std::fmt::Write;
 use std::io::{stderr, stdout, Stderr, Stdout, Write as WriteIo};
@@ -23,6 +22,7 @@ pub enum OperationType {
     CalculateDiff,
     CalculatePatchId,
     CheckForCycles,
+    ConstrainCommits,
     DetectDuplicateCommits,
     FilterByTouchedPaths,
     FilterCommits,
@@ -33,7 +33,10 @@ pub enum OperationType {
     InitializeRebase,
     MakeGraph,
     ProcessEvents,
+    ReadingFromCache,
+    RebaseCommits,
     RunGitCommand(Arc<String>),
+    SyncCommits,
     UpdateCommitGraph,
     WalkCommits,
 }
@@ -45,6 +48,7 @@ impl ToString for OperationType {
             OperationType::CalculateDiff => "Computing diffs",
             OperationType::CalculatePatchId => "Hashing commit contents",
             OperationType::CheckForCycles => "Checking for cycles",
+            OperationType::ConstrainCommits => "Creating commit constraints",
             OperationType::DetectDuplicateCommits => "Checking for duplicate commits",
             OperationType::FilterByTouchedPaths => "Filtering upstream commits by touched paths",
             OperationType::FilterCommits => "Filtering commits",
@@ -55,9 +59,12 @@ impl ToString for OperationType {
             OperationType::InitializeRebase => "Initializing rebase",
             OperationType::MakeGraph => "Examining local history",
             OperationType::ProcessEvents => "Processing events",
+            OperationType::ReadingFromCache => "Reading from cache",
+            OperationType::RebaseCommits => "Rebasing commits",
             OperationType::RunGitCommand(command) => {
                 return format!("Running Git command: {}", &command)
             }
+            OperationType::SyncCommits => "Syncing commit stacks",
             OperationType::UpdateCommitGraph => "Updating commit graph",
             OperationType::WalkCommits => "Walking commits",
         };
@@ -75,6 +82,142 @@ enum OutputDest {
     },
 }
 
+/// An index into the recursive hierarchy of progress bars. For example, the key
+/// `[OperationType::GetMergeBase, OperationType::WalkCommits]` refers to the
+/// "walk commits" operation which is nested under the "get merge-base"
+/// operation.
+type OperationKey = [OperationType];
+
+#[derive(Debug, Default)]
+struct RootOperation {
+    multi_progress: MultiProgress,
+    children: Vec<OperationState>,
+}
+
+impl RootOperation {
+    pub fn hide_multi_progress(&mut self) {
+        self.multi_progress
+            .set_draw_target(ProgressDrawTarget::hidden());
+    }
+
+    pub fn show_multi_progress(&mut self) {
+        self.multi_progress
+            .set_draw_target(ProgressDrawTarget::stderr());
+    }
+
+    /// If all operations are no longer in progress, clear the multi-progress bar.
+    pub fn clear_operations_if_finished(&mut self) {
+        if self
+            .children
+            .iter()
+            .all(|operation_state| operation_state.start_times.is_empty())
+        {
+            if self.multi_progress.clear().is_err() {
+                // Ignore error. Assume that the draw target is no longer available
+                // to write to.
+            }
+            self.children.clear();
+        }
+    }
+
+    pub fn get_or_create_child(&mut self, key: &[OperationType]) -> &mut OperationState {
+        match key {
+            [] => panic!("Empty operation key"),
+            [first, rest @ ..] => {
+                let index = match self
+                    .children
+                    .iter()
+                    .find_position(|child| &child.operation_type == first)
+                {
+                    Some((child_index, _)) => child_index,
+                    None => {
+                        self.children.push(OperationState {
+                            operation_type: first.clone(),
+                            progress_bar: ProgressBar::new_spinner(),
+                            has_meter: Default::default(),
+                            start_times: Default::default(),
+                            elapsed_duration: Default::default(),
+                            children: Default::default(),
+                        });
+                        self.children.len() - 1
+                    }
+                };
+                self.children
+                    .get_mut(index)
+                    .unwrap()
+                    .get_or_create_child(rest)
+            }
+        }
+    }
+
+    pub fn get_child(&mut self, key: &[OperationType]) -> Option<&mut OperationState> {
+        match key {
+            [] => panic!("Empty operation key"),
+            [first, rest @ ..] => {
+                let index = self
+                    .children
+                    .iter()
+                    .find_position(|child| &child.operation_type == first);
+                match index {
+                    Some((index, _)) => self.children.get_mut(index).unwrap().get_child(rest),
+                    None => None,
+                }
+            }
+        }
+    }
+
+    /// Re-render all operation progress bars. This does not change their
+    /// ordering like [`refresh_multi_progress`] does.
+    pub fn tick(&mut self) {
+        let operations = {
+            let mut acc = Vec::new();
+            self.traverse_operations(&mut acc, 0, &self.children);
+            acc
+        };
+        for (nesting_level, operation) in operations {
+            operation.tick(nesting_level);
+        }
+    }
+
+    /// Update the ordering of progress bars in the multi-progress. This should be called after
+    pub fn refresh_multi_progress(&mut self) {
+        let operations = {
+            let mut acc = Vec::new();
+            self.traverse_operations(&mut acc, 0, &self.children);
+            acc
+        };
+        if self.multi_progress.clear().is_err() {
+            // Ignore the error and assume that the multi-progress is now dead,
+            // so it doesn't need to be updated.
+        }
+        for (nesting_level, operation) in operations {
+            // Avoid deadlock inside the progress bar library when we call
+            // `add`, which sets the draw target again.
+            operation
+                .progress_bar
+                .set_draw_target(ProgressDrawTarget::hidden());
+
+            self.multi_progress.add(operation.progress_bar.clone());
+
+            // Re-render only after it's been added to the multi-progress, so
+            // that the draw target has been set.
+            operation.tick(nesting_level);
+        }
+    }
+
+    fn traverse_operations<'a>(
+        &self,
+        acc: &mut Vec<(usize, &'a OperationState)>,
+        current_level: usize,
+        operations: &'a [OperationState],
+    ) {
+        for operation in operations {
+            acc.push((current_level, operation));
+            self.traverse_operations(acc, current_level + 1, &operation.children);
+        }
+    }
+}
+
 #[derive(Debug)]
 struct OperationState {
     operation_type: OperationType,
@@ -82,11 +225,23 @@ struct OperationState {
     has_meter: bool,
     start_times: Vec<Instant>,
     elapsed_duration: Duration,
+    children: Vec<OperationState>,
 }
 
 impl OperationState {
     pub fn set_progress(&mut self, current: usize, total: usize) {
         self.has_meter = true;
+
+        if current
+            .try_into()
+            .map(|current: u64| current < self.progress_bar.position())
+            .unwrap_or(false)
+        {
+            // Workaround for issue fixed by
+            // <https://github.com/console-rs/indicatif/pull/403>.
+            self.progress_bar.reset_eta();
+        }
+
         self.progress_bar.set_position(current.try_into().unwrap());
         self.progress_bar.set_length(total.try_into().unwrap());
     }
@@ -95,13 +250,63 @@ impl OperationState {
         self.progress_bar.inc(increment.try_into().unwrap());
     }
 
-    pub fn tick(&self) {
+    pub fn get_or_create_child(&mut self, child_type: &[OperationType]) -> &mut Self {
+        match child_type {
+            [] => self,
+            [first, rest @ ..] => {
+                let index = match self
+                    .children
+                    .iter()
+                    .find_position(|child| &child.operation_type == first)
+                {
+                    Some((child_index, _)) => child_index,
+                    None => {
+                        self.children.push(OperationState {
+                            operation_type: first.clone(),
+                            progress_bar: ProgressBar::new_spinner(),
+                            has_meter: Default::default(),
+                            start_times: Default::default(),
+                            elapsed_duration: Default::default(),
+                            children: Default::default(),
+                        });
+                        self.children.len() - 1
+                    }
+                };
+                self.children
+                    .get_mut(index)
+                    .unwrap()
+                    .get_or_create_child(rest)
+            }
+        }
+    }
+
+    pub fn get_child(&mut self, child_type: &[OperationType]) -> Option<&mut Self> {
+        match child_type {
+            [] => Some(self),
+            [first, rest @ ..] => {
+                let index = self
+                    .children
+                    .iter()
+                    .find_position(|child| &child.operation_type == first);
+                match index {
+                    Some((index, _)) => self.children.get_mut(index).unwrap().get_child(rest),
+                    None => None,
+                }
+            }
+        }
+    }
+
+    pub fn tick(&self, nesting_level: usize) {
         lazy_static! {
             static ref CHECKMARK: String = console::style("✓").green().to_string();
             static ref IN_PROGRESS_SPINNER_STYLE: ProgressStyle =
                 ProgressStyle::default_spinner().template("{prefix}{spinner} {wide_msg}");
             static ref IN_PROGRESS_BAR_STYLE: ProgressStyle =
-                ProgressStyle::default_bar().template("{prefix}{spinner} {wide_msg} {bar} {pos}/{len}");
+                // indicatif places the cursor at the end of the line, which may
+                // be visible in the terminal, so we add a space at the end of
+                // the line so that the length number isn't overlapped by the
+                // cursor.
+                ProgressStyle::default_bar().template("{prefix}{spinner} {wide_msg} {bar} {pos}/{len} ");
             static ref FINISHED_PROGRESS_STYLE: ProgressStyle = IN_PROGRESS_SPINNER_STYLE
                 .clone()
                 // Requires at least two tick values, so just pass the same one twice.
@@ -116,18 +321,20 @@ impl OperationState {
             }
         };
 
-        self.progress_bar.set_message(format!(
-            "{} ({:.1}s)",
-            self.operation_type.to_string(),
-            elapsed_duration.as_secs_f64(),
-        ));
         self.progress_bar
             .set_style(match (self.start_times.as_slice(), self.has_meter) {
                 ([], _) => FINISHED_PROGRESS_STYLE.clone(),
                 ([..], false) => IN_PROGRESS_SPINNER_STYLE.clone(),
                 ([..], true) => IN_PROGRESS_BAR_STYLE.clone(),
             });
-        self.progress_bar.tick();
+        // Both `set_message` and `set_prefix` implicitly call
+        // `ProgressBar::tick` and force a redraw.
+        self.progress_bar.set_prefix("  ".repeat(nesting_level));
+        self.progress_bar.set_message(format!(
+            "{} ({:.1}s)",
+            self.operation_type.to_string(),
+            elapsed_duration.as_secs_f64(),
+        ));
     }
 }
 
@@ -137,10 +344,9 @@ impl OperationState {
 pub struct Effects {
     glyphs: Glyphs,
     dest: OutputDest,
-    multi_progress: Arc<MultiProgress>,
     updater_thread_handle: Arc<RwLock<UpdaterThreadHandle>>,
-    nesting_level: usize,
-    operation_states: Arc<RwLock<HashMap<OperationType, OperationState>>>,
+    operation_key: Vec<OperationType>,
+    root_operation: Arc<Mutex<RootOperation>>,
 }
 
 impl std::fmt::Debug for Effects {
@@ -159,12 +365,13 @@ struct UpdaterThreadHandle {
 }
 
 fn spawn_progress_updater_thread(
-    multi_progress: &Arc<MultiProgress>,
-    operation_states: &Arc<RwLock<HashMap<OperationType, OperationState>>>,
+    root_operation: &Arc<Mutex<RootOperation>>,
 ) -> Arc<RwLock<UpdaterThreadHandle>> {
-    multi_progress.set_draw_target(ProgressDrawTarget::hidden());
-    let multi_progress = Arc::downgrade(multi_progress);
-    let operation_states = Arc::downgrade(operation_states);
+    {
+        let mut root_operation = root_operation.lock().unwrap();
+        root_operation.hide_multi_progress();
+    }
+    let root_operation = Arc::downgrade(root_operation);
     let handle = Arc::new(RwLock::new(UpdaterThreadHandle { is_visible: false }));
 
     thread::spawn({
@@ -175,9 +382,10 @@ fn spawn_progress_updater_thread(
             thread::sleep(Duration::from_millis(250));
             {
                 let mut handle = handle.write().unwrap();
-                match multi_progress.upgrade() {
-                    Some(multi_progress) => {
-                        multi_progress.set_draw_target(ProgressDrawTarget::stderr())
+                match root_operation.upgrade() {
+                    Some(root_operation) => {
+                        let mut root_operation = root_operation.lock().unwrap();
+                        root_operation.show_multi_progress();
                     }
                     None => return,
                 }
@@ -187,13 +395,11 @@ fn spawn_progress_updater_thread(
             loop {
                 // Drop the `Arc` after this block, before the sleep, to make sure
                 // that progress bars aren't kept alive longer than they should be.
-                match operation_states.upgrade() {
+                match root_operation.upgrade() {
                     None => return,
-                    Some(operation_states) => {
-                        let operation_states = operation_states.read().unwrap();
-                        for operation_state in operation_states.values() {
-                            operation_state.tick();
-                        }
+                    Some(root_operation) => {
+                        let mut root_operation = root_operation.lock().unwrap();
+                        root_operation.tick();
                     }
                 }
 
@@ -208,17 +414,14 @@ fn spawn_progress_updater_thread(
 impl Effects {
     /// Constructor. Writes to stdout.
     pub fn new(glyphs: Glyphs) -> Self {
-        let multi_progress = Default::default();
-        let operation_states = Default::default();
-        let updater_thread_handle =
-            spawn_progress_updater_thread(&multi_progress, &operation_states);
+        let root_operation = Default::default();
+        let updater_thread_handle = spawn_progress_updater_thread(&root_operation);
         Effects {
             glyphs,
             dest: OutputDest::Stdout,
-            multi_progress,
             updater_thread_handle,
-            nesting_level: Default::default(),
-            operation_states,
+            operation_key: Default::default(),
+            root_operation,
         }
     }
 
@@ -227,10 +430,9 @@ impl Effects {
         Effects {
             glyphs,
             dest: OutputDest::Suppress,
-            multi_progress: Default::default(),
             updater_thread_handle: Default::default(),
-            nesting_level: Default::default(),
-            operation_states: Default::default(),
+            operation_key: Default::default(),
+            root_operation: Default::default(),
         }
     }
 
@@ -246,18 +448,17 @@ impl Effects {
                 stdout: Arc::clone(stdout),
                 stderr: Arc::clone(stderr),
             },
-            multi_progress: Default::default(),
             updater_thread_handle: Default::default(),
-            nesting_level: Default::default(),
-            operation_states: Default::default(),
+            operation_key: Default::default(),
+            root_operation: Default::default(),
         }
     }
 
     /// Send output to an appropriate place when using a terminal user interface
     /// (TUI), such as for `git undo`.
     pub fn enable_tui_mode(&self) -> Self {
-        let multi_progress = Arc::clone(&self.multi_progress);
-        multi_progress.set_draw_target(ProgressDrawTarget::hidden());
+        let mut root_operation = self.root_operation.lock().unwrap();
+        root_operation.hide_multi_progress();
         Self {
             dest: OutputDest::Suppress,
             ..self.clone()
@@ -284,9 +485,14 @@ impl Effects {
     /// see the aggregate time it took to carry out sibling operations, i.e. the
     /// same operation called multiple times in a loop.
     pub fn start_operation(&self, operation_type: OperationType) -> (Effects, ProgressHandle) {
+        let operation_key = {
+            let mut result = self.operation_key.clone();
+            result.push(operation_type);
+            result
+        };
         let progress = ProgressHandle {
             effects: self,
-            operation_type: operation_type.clone(),
+            operation_key: operation_key.clone(),
         };
         match self.dest {
             OutputDest::Stdout => {}
@@ -296,64 +502,52 @@ impl Effects {
         }
 
         let now = Instant::now();
-        let mut operation_states = self.operation_states.write().unwrap();
-
-        let mut nesting_level = self.nesting_level;
-        let operation_state = operation_states
-            .entry(operation_type.clone())
-            .or_insert_with(|| {
-                let progress_bar = self.multi_progress.add(ProgressBar::new_spinner());
-                nesting_level += 1;
-                progress_bar.set_prefix("  ".repeat(nesting_level));
-                let operation_state = OperationState {
-                    operation_type,
-                    progress_bar,
-                    start_times: Vec::new(),
-                    has_meter: false,
-                    elapsed_duration: Default::default(),
-                };
-                operation_state.tick();
-                operation_state
-            });
+        let mut root_operation = self.root_operation.lock().unwrap();
+        let operation_state = root_operation.get_or_create_child(&operation_key);
         operation_state.start_times.push(now);
+        root_operation.refresh_multi_progress();
 
         let effects = Self {
-            nesting_level,
+            operation_key,
             ..self.clone()
         };
         (effects, progress)
     }
 
-    fn on_notify_progress(&self, operation_type: OperationType, current: usize, total: usize) {
-        let mut operation_states = self.operation_states.write().unwrap();
-        let operation_state = match operation_states.get_mut(&operation_type) {
-            Some(operation_state) => operation_state,
-            None => return,
-        };
-
+    fn on_notify_progress(&self, operation_key: &OperationKey, current: usize, total: usize) {
+        let mut root_operation = self.root_operation.lock().unwrap();
+        let operation_state = root_operation.get_or_create_child(operation_key);
         operation_state.set_progress(current, total);
     }
 
-    fn on_notify_progress_inc(&self, operation_type: OperationType, increment: usize) {
-        let mut operation_states = self.operation_states.write().unwrap();
-        let operation_state = match operation_states.get_mut(&operation_type) {
+    fn on_notify_progress_inc(&self, operation_key: &OperationKey, increment: usize) {
+        let mut root_operation = self.root_operation.lock().unwrap();
+        let operation_state = match root_operation.get_child(operation_key) {
             Some(operation_state) => operation_state,
             None => return,
         };
-
         operation_state.inc_progress(increment);
     }
 
-    fn on_drop_progress_handle(&self, operation_type: OperationType) {
+    fn on_set_message(&self, operation_key: &OperationKey, message: String) {
+        let mut root_operation = self.root_operation.lock().unwrap();
+        let operation_state = match root_operation.get_child(operation_key) {
+            Some(operation_state) => operation_state,
+            None => return,
+        };
+        operation_state.progress_bar.set_message(message);
+    }
+
+    fn on_drop_progress_handle(&self, operation_key: &OperationKey) {
         match self.dest {
             OutputDest::Stdout => {}
             OutputDest::Suppress | OutputDest::BufferForTest { .. } => return,
         }
 
         let now = Instant::now();
-        let mut operation_states = self.operation_states.write().unwrap();
+        let mut root_operation = self.root_operation.lock().unwrap();
 
-        let operation_state = match operation_states.get_mut(&operation_type) {
+        let operation_state = match root_operation.get_child(operation_key) {
             Some(operation_state) => operation_state,
             None => {
                 warn!("Progress operation not started");
@@ -384,14 +578,7 @@ impl Effects {
             Duration::ZERO
         };
 
-        // Reset all elapsed times only if the root-level operation completed.
-        if operation_states
-            .values()
-            .all(|operation_state| operation_state.start_times.is_empty())
-        {
-            self.multi_progress.clear().unwrap();
-            operation_states.clear();
-        }
+        root_operation.clear_operations_if_finished();
     }
 
     /// Get the set of glyphs associated with the output.
@@ -406,7 +593,7 @@ impl Effects {
             dest: self.dest.clone(),
             buffer: Default::default(),
             updater_thread_handle: Arc::clone(&self.updater_thread_handle),
-            operation_states: Arc::clone(&self.operation_states),
+            root_operation: Arc::clone(&self.root_operation),
         }
     }
 
@@ -417,7 +604,7 @@ impl Effects {
             dest: self.dest.clone(),
             buffer: Default::default(),
             updater_thread_handle: Arc::clone(&self.updater_thread_handle),
-            operation_states: Arc::clone(&self.operation_states),
+            root_operation: Arc::clone(&self.root_operation),
         }
     }
 }
@@ -426,19 +613,19 @@ trait WriteProgress {
     type Stream: WriteIo;
     fn get_stream() -> Self::Stream;
     fn get_buffer(&mut self) -> &mut String;
-    fn get_operation_states(&self) -> Arc<RwLock<HashMap<OperationType, OperationState>>>;
+    fn get_root_operation(&self) -> Arc<Mutex<RootOperation>>;
     fn get_updater_thread_handle(&self) -> Arc<RwLock<UpdaterThreadHandle>>;
     fn style_output(output: &str) -> String;
 
     fn flush(&mut self) {
-        let operation_states = self.get_operation_states();
-        let operation_states = operation_states.read().unwrap();
+        let root_operation = self.get_root_operation();
+        let root_operation = root_operation.lock().unwrap();
 
         // Get an arbitrary progress meter. It turns out that when a
         // `ProgressBar` is included in a `MultiProgress`, it doesn't matter
         // which of them we call `println` on. The output will be printed above
         // the `MultiProgress` regardless.
-        match operation_states.values().next() {
+        match root_operation.children.get(0) {
             None => {
                 // There's no progress meters, so we can write directly to
                 // stdout. Note that we don't style output here; instead, we
@@ -483,7 +670,7 @@ trait WriteProgress {
                 // full lines at a time.
                 *self.get_buffer() = {
                     let mut new_buffer = String::new();
-                    let lines = self.get_buffer().split_inclusive("\n");
+                    let lines = self.get_buffer().split_inclusive('\n');
                     for line in lines {
                         match line.strip_suffix('\n') {
                             Some(line) => operation_state
@@ -498,7 +685,7 @@ trait WriteProgress {
                     new_buffer
                 };
             }
-        }
+        };
     }
 
     fn drop(&mut self) {
@@ -525,7 +712,7 @@ pub struct OutputStream {
     dest: OutputDest,
     buffer: String,
     updater_thread_handle: Arc<RwLock<UpdaterThreadHandle>>,
-    operation_states: Arc<RwLock<HashMap<OperationType, OperationState>>>,
+    root_operation: Arc<Mutex<RootOperation>>,
 }
 
 impl WriteProgress for OutputStream {
@@ -539,8 +726,8 @@ impl WriteProgress for OutputStream {
         &mut self.buffer
     }
 
-    fn get_operation_states(&self) -> Arc<RwLock<HashMap<OperationType, OperationState>>> {
-        Arc::clone(&self.operation_states)
+    fn get_root_operation(&self) -> Arc<Mutex<RootOperation>> {
+        Arc::clone(&self.root_operation)
     }
 
     fn get_updater_thread_handle(&self) -> Arc<RwLock<UpdaterThreadHandle>> {
@@ -548,6 +735,7 @@ impl WriteProgress for OutputStream {
     }
 
     fn style_output(output: &str) -> String {
+        let output = console::strip_ansi_codes(output);
         console::style(output).dim().to_string()
     }
 }
@@ -584,7 +772,7 @@ pub struct ErrorStream {
     dest: OutputDest,
     buffer: String,
     updater_thread_handle: Arc<RwLock<UpdaterThreadHandle>>,
-    operation_states: Arc<RwLock<HashMap<OperationType, OperationState>>>,
+    root_operation: Arc<Mutex<RootOperation>>,
 }
 
 impl WriteProgress for ErrorStream {
@@ -598,8 +786,8 @@ impl WriteProgress for ErrorStream {
         &mut self.buffer
     }
 
-    fn get_operation_states(&self) -> Arc<RwLock<HashMap<OperationType, OperationState>>> {
-        Arc::clone(&self.operation_states)
+    fn get_root_operation(&self) -> Arc<Mutex<RootOperation>> {
+        Arc::clone(&self.root_operation)
     }
 
     fn get_updater_thread_handle(&self) -> Arc<RwLock<UpdaterThreadHandle>> {
@@ -607,6 +795,7 @@ impl WriteProgress for ErrorStream {
     }
 
     fn style_output(output: &str) -> String {
+        let output = console::strip_ansi_codes(output);
         console::style(output).dim().to_string()
     }
 }
@@ -641,15 +830,15 @@ impl Drop for ErrorStream {
 /// A handle to an operation in progress. This object should be kept live while
 /// the operation is underway, and a timing entry for it will be displayed in
 /// the interactive progress display.
+#[derive(Debug)]
 pub struct ProgressHandle<'a> {
     effects: &'a Effects,
-    operation_type: OperationType,
+    operation_key: Vec<OperationType>,
 }
 
 impl Drop for ProgressHandle<'_> {
     fn drop(&mut self) {
-        self.effects
-            .on_drop_progress_handle(self.operation_type.clone())
+        self.effects.on_drop_progress_handle(&self.operation_key)
     }
 }
 
@@ -659,7 +848,7 @@ impl ProgressHandle<'_> {
     /// through the operation.
     pub fn notify_progress(&self, current: usize, total: usize) {
         self.effects
-            .on_notify_progress(self.operation_type.clone(), current, total);
+            .on_notify_progress(&self.operation_key, current, total);
     }
 
     /// Notify the progress meter that additional progress has taken place.
@@ -667,7 +856,13 @@ impl ProgressHandle<'_> {
     /// much total work there is.
     pub fn notify_progress_inc(&self, increment: usize) {
         self.effects
-            .on_notify_progress_inc(self.operation_type.clone(), increment);
+            .on_notify_progress_inc(&self.operation_key, increment);
+    }
+
+    /// Update the message for this progress meter.
+    pub fn notify_status(&self, message: impl Into<String>) {
+        let message = message.into();
+        self.effects.on_set_message(&self.operation_key, message);
     }
 }
 
@@ -681,27 +876,30 @@ mod tests {
         let (effects2, progress2) = effects.start_operation(OperationType::GetMergeBase);
 
         {
-            let operation_states = effects.operation_states.read().unwrap();
-            let get_merge_base_operation =
-                operation_states.get(&OperationType::GetMergeBase).unwrap();
+            let mut root_operation = effects.root_operation.lock().unwrap();
+            let get_merge_base_operation = root_operation
+                .get_child(&[OperationType::GetMergeBase])
+                .unwrap();
             assert_eq!(get_merge_base_operation.start_times.len(), 1);
         }
 
         std::thread::sleep(Duration::from_millis(1));
         let (_effects3, progress3) = effects.start_operation(OperationType::GetMergeBase);
         let earlier_start_time = {
-            let operation_states = effects.operation_states.read().unwrap();
-            let get_merge_base_operation =
-                operation_states.get(&OperationType::GetMergeBase).unwrap();
+            let mut root_operation = effects.root_operation.lock().unwrap();
+            let get_merge_base_operation = root_operation
+                .get_child(&[OperationType::GetMergeBase])
+                .ok_or_else(|| eyre::eyre!("Could not find merge-base operation"))?;
             assert_eq!(get_merge_base_operation.start_times.len(), 2);
             get_merge_base_operation.start_times[0]
         };
 
         drop(progress3);
         {
-            let operation_states = effects.operation_states.read().unwrap();
-            let get_merge_base_operation =
-                operation_states.get(&OperationType::GetMergeBase).unwrap();
+            let mut root_operation = effects.root_operation.lock().unwrap();
+            let get_merge_base_operation = root_operation
+                .get_child(&[OperationType::GetMergeBase])
+                .unwrap();
             // Ensure that we try to keep the earliest times in the list, to
             // accurately gauge the wall-clock time.
             assert_eq!(
@@ -715,21 +913,34 @@ mod tests {
         std::thread::sleep(Duration::from_millis(1));
         drop(progress4);
         {
-            let operation_states = effects.operation_states.read().unwrap();
+            let mut root_operation = effects.root_operation.lock().unwrap();
             // The operation should still be present until the root-level
             // operation has finished, even if it's not currently in progress.
-            let calculate_diff_operation =
-                operation_states.get(&OperationType::CalculateDiff).unwrap();
+            let calculate_diff_operation = root_operation
+                .get_child(&[OperationType::GetMergeBase, OperationType::CalculateDiff])
+                .unwrap();
             assert!(calculate_diff_operation.start_times.is_empty());
             assert!(calculate_diff_operation.elapsed_duration >= Duration::from_millis(1));
         }
 
         drop(progress2);
         {
-            let operation_states = effects.operation_states.read().unwrap();
-            assert!(operation_states.is_empty());
+            let root_operation = effects.root_operation.lock().unwrap();
+            assert!(root_operation.children.is_empty());
         }
 
         Ok(())
     }
+
+    /// Test for the issue fixed by <https://github.com/console-rs/indicatif/pull/403>.
+    #[test]
+    fn test_effects_progress_rewind_panic() -> eyre::Result<()> {
+        let effects = Effects::new(Glyphs::text());
+        let (effects, progress) = effects.start_operation(OperationType::GetMergeBase);
+        let _ = effects;
+        progress.notify_progress(3, 10);
+        // Should not panic.
+        progress.notify_progress(0, 10);
+        Ok(())
+    }
 }
