@@ -4,8 +4,9 @@ use eden_dag::DagAlgorithm;
 use itertools::Itertools;
 use tracing::instrument;
 
+use crate::core::dag::{CommitSet, Dag};
 use crate::core::eventlog::{Event, EventCursor, EventReplayer};
-use crate::git::{CommitSet, Dag, MaybeZeroOid, NonZeroOid};
+use crate::git::{MaybeZeroOid, NonZeroOid};
 
 /// For a rewritten commit, find the newest version of the commit.
 ///
@@ -15,6 +16,7 @@ use crate::git::{CommitSet, Dag, MaybeZeroOid, NonZeroOid};
 ///
 /// If a commit was rewritten into itself through some chain of events, then
 /// returns `None`, rather than the same commit OID.
+#[instrument]
 pub fn find_rewrite_target(
     event_replayer: &EventReplayer,
     event_cursor: EventCursor,
@@ -80,6 +82,7 @@ pub fn find_abandoned_children(
     };
 
     let children = dag.query().children(CommitSet::from(oid))?;
+    let children = children.intersection(&dag.observed_commits);
     let non_obsolete_children = children.difference(&dag.obsolete_commits);
     let non_obsolete_children_oids: Vec<NonZeroOid> = non_obsolete_children
         .iter()?
@@ -91,10 +94,10 @@ pub fn find_abandoned_children(
 
 #[cfg(test)]
 mod tests {
+    use crate::core::effects::Effects;
     use crate::core::eventlog::EventLogDb;
     use crate::core::formatting::Glyphs;
     use crate::testing::{make_git, Git, GitRunOptions};
-    use crate::tui::Effects;
 
     use super::*;
 
