@@ -3,18 +3,19 @@
 //! This is inside `src` rather than `tests` since we use this code in some unit
 //! tests.
 
-use std::collections::HashMap;
-use std::ffi::{OsStr, OsString};
+use std::collections::{BTreeMap, HashMap};
+use std::ffi::OsString;
+use std::fs;
 use std::io::Write;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 
-use crate::git::{GitRunInfo, GitVersion, NonZeroOid, Repo};
+use crate::core::config::env_vars::{get_git_exec_path, get_path_to_git, TEST_GIT};
+use crate::git::{GitVersion, NonZeroOid, Repo};
 use crate::util::get_sh;
 
-use color_eyre::Help;
-use eyre::{eyre, Context};
+use eyre::Context;
 use itertools::Itertools;
 use lazy_static::lazy_static;
 use regex::{Captures, Regex};
@@ -36,6 +37,9 @@ pub struct Git {
     /// The path to the Git executable on disk. This is important since we test
     /// against multiple Git versions.
     pub path_to_git: PathBuf,
+
+    /// The `GIT_EXEC_PATH` environment variable value to use for testing.
+    pub git_exec_path: PathBuf,
 }
 
 /// Options for `Git::init_repo_with_options`.
@@ -77,17 +81,11 @@ pub struct GitRunOptions {
 
 impl Git {
     /// Constructor.
-    pub fn new(repo_path: PathBuf, git_run_info: GitRunInfo) -> Self {
-        let GitRunInfo {
-            path_to_git,
-            // We pass the repo directory when calling `run`.
-            working_directory: _,
-            // We manually set the environment when calling `run`.
-            env: _,
-        } = git_run_info;
+    pub fn new(path_to_git: PathBuf, repo_path: PathBuf, git_exec_path: PathBuf) -> Self {
         Git {
             repo_path,
             path_to_git,
+            git_exec_path,
         }
     }
 
@@ -130,19 +128,6 @@ impl Git {
         Ok(output)
     }
 
-    fn get_git_exec_path(&self) -> PathBuf {
-        match std::env::var_os("GIT_EXEC_PATH") {
-            Some(git_exec_path) => git_exec_path.into(),
-            None => {
-                let git_path = self
-                    .path_to_git
-                    .parent()
-                    .expect("Unable to find git path parent");
-                git_path.to_path_buf()
-            }
-        }
-    }
-
     /// Get the `PATH` environment variable to use for testing.
     pub fn get_path_for_env(&self) -> OsString {
         let cargo_bin_path = assert_cmd::cargo::cargo_bin("git-branchless");
@@ -156,7 +141,7 @@ impl Git {
                 // For Git to be able to launch `git-branchless`.
                 branchless_path.as_os_str(),
                 // For our hooks to be able to call back into `git`.
-                self.get_git_exec_path().as_os_str(),
+                self.git_exec_path.as_os_str(),
                 // For branchless to manually invoke bash when needed.
                 bash_path.as_os_str(),
             ]
@@ -165,6 +150,36 @@ impl Git {
         .expect("joining paths")
     }
 
+    /// Get the environment variables needed to run git in the test environment.
+    pub fn get_base_env(&self, time: isize) -> Vec<(OsString, OsString)> {
+        // Required for determinism, as these values will be baked into the commit
+        // hash.
+        let date: OsString = format!("{date} -{time:0>2}", date = DUMMY_DATE, time = time).into();
+
+        // Fake "editor" which accepts the default contents of any commit
+        // messages. Usually, we can set this with `git commit -m`, but we have
+        // no such option for things such as `git rebase`, which may call `git
+        // commit` later as a part of their execution.
+        //
+        // ":" is understood by `git` to skip editing.
+        let git_editor = OsString::from(":");
+
+        let new_path = self.get_path_for_env();
+        let envs = vec![
+            ("GIT_CONFIG_NOSYSTEM", OsString::from("1")),
+            ("GIT_AUTHOR_DATE", date.clone()),
+            ("GIT_COMMITTER_DATE", date),
+            ("GIT_EDITOR", git_editor),
+            ("GIT_EXEC_PATH", self.git_exec_path.as_os_str().into()),
+            ("PATH", new_path),
+            (TEST_GIT, self.path_to_git.as_os_str().into()),
+        ];
+
+        envs.into_iter()
+            .map(|(key, value)| (OsString::from(key), value))
+            .collect()
+    }
+
     #[instrument]
     fn run_with_options_inner(
         &self,
@@ -178,42 +193,20 @@ impl Git {
             env,
         } = options;
 
-        // Required for determinism, as these values will be baked into the commit
-        // hash.
-        let date: OsString = format!("{date} -{time:0>2}", date = DUMMY_DATE, time = time).into();
-
-        let args: Vec<&str> = {
-            let repo_path = self.repo_path.to_str().expect("Could not decode repo path");
-            let mut new_args: Vec<&str> = vec!["-C", repo_path];
-            new_args.extend(args);
-            new_args
-        };
-
-        // Fake "editor" which accepts the default contents of any commit
-        // messages. Usually, we can set this with `git commit -m`, but we have
-        // no such option for things such as `git rebase`, which may call `git
-        // commit` later as a part of their execution.
-        //
-        // ":" is understood by `git` to skip editing.
-        let git_editor = OsStr::new(":");
-
-        let git_exec_path = self.get_git_exec_path();
-        let new_path = self.get_path_for_env();
-        let base_env: Vec<(&str, &OsStr)> = vec![
-            ("GIT_AUTHOR_DATE", &date),
-            ("GIT_COMMITTER_DATE", &date),
-            ("GIT_EDITOR", git_editor),
-            ("GIT_EXEC_PATH", git_exec_path.as_os_str()),
-            ("PATH_TO_GIT", self.path_to_git.as_os_str()),
-            ("PATH", &new_path),
-        ];
-
+        let env: BTreeMap<_, _> = self
+            .get_base_env(*time)
+            .into_iter()
+            .chain(
+                env.iter()
+                    .map(|(k, v)| (OsString::from(k), OsString::from(v))),
+            )
+            .collect();
         let mut command = Command::new(&self.path_to_git);
         command
-            .args(&args)
+            .current_dir(&self.repo_path)
+            .args(args)
             .env_clear()
-            .envs(base_env.iter().copied())
-            .envs(env.iter());
+            .envs(&env);
 
         let result = if let Some(input) = input {
             let mut child = command
@@ -251,14 +244,17 @@ impl Git {
         let result = if exit_code != *expected_exit_code {
             eyre::bail!(
                 "Git command {:?} {:?} exited with unexpected code {} (expected {})
-                stdout:
-                {}
-                stderr:
-                {}",
+env:
+{:#?}
+stdout:
+{}
+stderr:
+{}",
                 &self.path_to_git,
                 &args,
                 exit_code,
                 expected_exit_code,
+                &env,
                 &String::from_utf8_lossy(&result.stdout),
                 &String::from_utf8_lossy(&result.stderr),
             )
@@ -305,7 +301,12 @@ impl Git {
         }
 
         // Non-deterministic metadata (depends on current time).
-        self.run(&["config", "branchless.commitMetadata.relativeTime", "false"])?;
+        self.run(&[
+            "config",
+            "branchless.commitDescriptors.relativeTime",
+            "false",
+        ])?;
+        self.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
         // Disable warnings of the following form on Windows:
         //
@@ -359,6 +360,24 @@ impl Git {
         Ok(())
     }
 
+    /// Delete the provided file in the repository root.
+    pub fn delete_file(&self, name: &str) -> eyre::Result<()> {
+        let file_path = self.repo_path.join(format!("{}.txt", name));
+        fs::remove_file(file_path)?;
+        Ok(())
+    }
+
+    /// Delete the provided file in the repository root.
+    pub fn set_file_permissions(
+        &self,
+        name: &str,
+        permissions: fs::Permissions,
+    ) -> eyre::Result<()> {
+        let file_path = self.repo_path.join(format!("{}.txt", name));
+        fs::set_permissions(file_path, permissions)?;
+        Ok(())
+    }
+
     /// Commit a file with default contents. The `time` argument is used to set
     /// the commit timestamp, which is factored into the commit hash.
     #[instrument]
@@ -448,6 +467,13 @@ impl Git {
         Ok(version >= GitVersion(2, 29, 0))
     }
 
+    /// The `log.excludeDecoration` configuration option was introduced in Git
+    /// v2.27.
+    pub fn supports_log_exclude_decoration(&self) -> eyre::Result<bool> {
+        let version = self.get_version()?;
+        Ok(version >= GitVersion(2, 27, 0))
+    }
+
     /// Resolve a file during a merge or rebase conflict with the provided
     /// contents.
     #[instrument]
@@ -463,18 +489,6 @@ impl Git {
     }
 }
 
-/// Get the path to the Git executable for testing.
-#[instrument]
-pub fn get_path_to_git() -> eyre::Result<PathBuf> {
-    let path_to_git = std::env::var_os("PATH_TO_GIT")
-        .ok_or_else(|| eyre!("Could not get path to Git executable"))
-        .suggestion(
-            "No path to git set. Try running as: PATH_TO_GIT=$(which git) cargo test ...",
-        )?;
-    let path_to_git = PathBuf::from(&path_to_git);
-    Ok(path_to_git)
-}
-
 /// Wrapper around a `Git` instance which cleans up the repository once dropped.
 pub struct GitWrapper {
     repo_dir: TempDir,
@@ -492,7 +506,6 @@ impl Deref for GitWrapper {
 /// From https://stackoverflow.com/a/65192210
 /// License: CC-BY-SA 4.0
 fn copy_dir_all(src: impl AsRef<Path>, dst: impl AsRef<Path>) -> std::io::Result<()> {
-    use std::fs;
     fs::create_dir_all(&dst)?;
     for entry in fs::read_dir(src)? {
         let entry = entry?;
@@ -517,7 +530,7 @@ impl GitWrapper {
         copy_dir_all(&self.repo_dir, &repo_dir)?;
         let git = Git {
             repo_path: repo_dir.path().to_path_buf(),
-            path_to_git: self.git.path_to_git.clone(),
+            ..self.git.clone()
         };
         Ok(Self { repo_dir, git })
     }
@@ -527,12 +540,8 @@ impl GitWrapper {
 pub fn make_git() -> eyre::Result<GitWrapper> {
     let repo_dir = tempfile::tempdir()?;
     let path_to_git = get_path_to_git()?;
-    let path_to_git = GitRunInfo {
-        path_to_git,
-        working_directory: repo_dir.path().to_path_buf(),
-        env: std::env::vars_os().collect(),
-    };
-    let git = Git::new(repo_dir.path().to_path_buf(), path_to_git);
+    let git_exec_path = get_git_exec_path()?;
+    let git = Git::new(path_to_git, repo_dir.path().to_path_buf(), git_exec_path);
     Ok(GitWrapper { repo_dir, git })
 }
 
@@ -551,32 +560,20 @@ pub struct GitWrapperWithRemoteRepo {
     pub cloned_repo: Git,
 }
 
-/// Create a `GitWrapperWithRemoteRepo`.
+/// Create a [`GitWrapperWithRemoteRepo`].
 pub fn make_git_with_remote_repo() -> eyre::Result<GitWrapperWithRemoteRepo> {
     let path_to_git = get_path_to_git()?;
+    let git_exec_path = get_git_exec_path()?;
     let temp_dir = tempfile::tempdir()?;
-    let git_run_info = GitRunInfo {
-        path_to_git,
-        working_directory: temp_dir.path().to_path_buf(),
-        env: Default::default(),
-    };
     let original_repo_path = temp_dir.path().join("original");
     std::fs::create_dir_all(&original_repo_path)?;
     let original_repo = Git::new(
-        original_repo_path.clone(),
-        GitRunInfo {
-            working_directory: original_repo_path,
-            ..git_run_info.clone()
-        },
+        path_to_git.clone(),
+        original_repo_path,
+        git_exec_path.clone(),
     );
     let cloned_repo_path = temp_dir.path().join("cloned");
-    let cloned_repo = Git::new(
-        cloned_repo_path.clone(),
-        GitRunInfo {
-            working_directory: cloned_repo_path,
-            ..git_run_info
-        },
-    );
+    let cloned_repo = Git::new(path_to_git, cloned_repo_path, git_exec_path);
 
     Ok(GitWrapperWithRemoteRepo {
         temp_dir,
