@@ -1,3 +1,5 @@
+use std::collections::HashMap;
+
 use regex::Regex;
 
 use crate::util::trim_lines;
@@ -52,7 +54,7 @@ echo Hello, world
         insta::assert_snapshot!(stderr, @r###"
         branchless: processing 2 updates: branch master, ref HEAD
         Hello, world
-        branchless: processed commit: 4cd1a9ba test
+        branchless: processed commit: 4cd1a9b test
         "###);
     }
 
@@ -67,16 +69,101 @@ fn test_alias_installed() -> eyre::Result<()> {
 
     {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
-        insta::assert_snapshot!(stdout, @r###"
-@ f777ecc9 (master) create initial.txt
-"###);
+        insta::assert_snapshot!(stdout, @"@ f777ecc (> master) create initial.txt
+");
     }
 
     {
         let (stdout, _stderr) = git.run(&["sl"])?;
-        insta::assert_snapshot!(stdout, @r###"
-@ f777ecc9 (master) create initial.txt
-"###);
+        insta::assert_snapshot!(stdout, @"@ f777ecc (> master) create initial.txt
+");
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_dont_install_existing_aliases() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    let git_init_options = GitInitOptions {
+        run_branchless_init: false,
+        ..Default::default()
+    };
+
+    git.init_repo_with_options(&git_init_options)?;
+
+    // Create a fake $HOME directory, to allow us to emulate a user's .gitconfig file
+    let fake_home_dir = git.repo_path.join("fake_home");
+    let fake_home_git_config = fake_home_dir.join(".gitconfig");
+    std::fs::create_dir(&fake_home_dir)?;
+    std::fs::write(&fake_home_git_config, "[alias]\n\tsl = status\n")?;
+
+    let env = HashMap::from([(
+        "HOME".to_string(),
+        fake_home_dir.to_string_lossy().to_string(),
+    )]);
+    let git_run_options = GitRunOptions {
+        env,
+        ..GitRunOptions::default()
+    };
+
+    // Initialize branchless and make sure it didn't add the "sl" alias
+    git.run_with_options(&["branchless", "init"], &git_run_options)?;
+
+    {
+        let (expected_stdout, _) = git.run_with_options(&["status"], &git_run_options)?;
+        let (actual_stdout, _) = git.run_with_options(&["sl"], &git_run_options)?;
+
+        assert_eq!(expected_stdout, actual_stdout);
+    }
+
+    // Update the config to add "smartlog" and make sure neither alias is added
+    std::fs::write(
+        &fake_home_git_config,
+        "[alias]\n\tsl = status\n\tsmartlog = status",
+    )?;
+
+    git.run_with_options(&["branchless", "init"], &git_run_options)?;
+
+    {
+        let (expected_stdout, _) = git.run_with_options(&["status"], &git_run_options)?;
+        let (actual_stdout, _) = git.run_with_options(&["sl"], &git_run_options)?;
+
+        assert_eq!(expected_stdout, actual_stdout);
+    }
+
+    {
+        let (expected_stdout, _) = git.run_with_options(&["status"], &git_run_options)?;
+        let (actual_stdout, _) = git.run_with_options(&["smartlog"], &git_run_options)?;
+
+        assert_eq!(expected_stdout, actual_stdout);
+    }
+
+    // Update the config to remove both aliases and make sure both are added
+    std::fs::write(&fake_home_git_config, "")?;
+
+    git.run_with_options(&["branchless", "init"], &git_run_options)?;
+
+    {
+        let (expected_stdout, _) = git.run_with_options(&["smartlog"], &git_run_options)?;
+        let (actual_stdout, _) = git.run_with_options(&["sl"], &git_run_options)?;
+
+        assert_eq!(expected_stdout, actual_stdout);
+    }
+
+    {
+        let (expected_stdout, _) = git.run_with_options(&["status"], &git_run_options)?;
+        let (actual_stdout, _) = git.run_with_options(&["sl"], &git_run_options)?;
+
+        assert_ne!(expected_stdout, actual_stdout);
+    }
+
+    {
+        let (expected_stdout, _) = git.run_with_options(&["status"], &git_run_options)?;
+        let (actual_stdout, _) = git.run_with_options(&["smartlog"], &git_run_options)?;
+
+        assert_ne!(expected_stdout, actual_stdout);
     }
 
     Ok(())
@@ -198,7 +285,7 @@ fn test_init_prompt_for_main_branch() -> eyre::Result<()> {
 
     {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
-        insta::assert_snapshot!(stdout, @"@ f777ecc9 (bespoke) create initial.txt
+        insta::assert_snapshot!(stdout, @"@ f777ecc (> bespoke) create initial.txt
 ");
     }
 
@@ -224,7 +311,7 @@ fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
         },
     )?;
 
-    let location_trace_re = Regex::new(r"[^ .]+\.rs:[0-9]+")?;
+    let location_trace_re = Regex::new(r"[^ ]+\.rs:[0-9]+")?;
     let stderr = trim_lines(stderr);
     let stderr = console::strip_ansi_codes(&stderr);
     let stderr = location_trace_re.replace_all(&stderr, "some/file/path.rs:123");
@@ -240,7 +327,7 @@ fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
 
        0: branchless::git::repo::get_main_branch_oid with self=<Git repository at: "<repo-path>/.git/">
           at some/file/path.rs:123
-       1: branchless::commands::smartlog::smartlog with effects=<Output fancy=false> git_run_info=<GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> options=SmartlogOptions { show_hidden_commits: false }
+       1: branchless::commands::smartlog::smartlog with effects=<Output fancy=false> git_run_info=<GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> options=SmartlogOptions { show_hidden_commits: false, only_show_branches: false }
           at some/file/path.rs:123
 
     Suggestion:
@@ -252,13 +339,11 @@ fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
         git config branchless.core.mainBranch <branch>
 
 
-    Backtrace omitted.
-    Run with RUST_BACKTRACE=1 environment variable to display it.
+    Backtrace omitted. Run with RUST_BACKTRACE=1 environment variable to display it.
     Run with RUST_BACKTRACE=full to include source snippets.
     Location: some/file/path.rs:123
 
-    Backtrace omitted.
-    Run with RUST_BACKTRACE=1 environment variable to display it.
+    Backtrace omitted. Run with RUST_BACKTRACE=1 environment variable to display it.
     Run with RUST_BACKTRACE=full to include source snippets.
     "###);
     insta::assert_snapshot!(stdout, @"");
@@ -382,7 +467,7 @@ fn test_init_explicit_main_branch_name() -> eyre::Result<()> {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
         :
-        @ 62fc20d2 (foo) create test1.txt
+        @ 62fc20d (> foo) create test1.txt
         "###);
     }
 
@@ -409,7 +494,7 @@ fn test_init_repo_default_branch() -> eyre::Result<()> {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
         :
-        @ 62fc20d2 (repo-default-branch) create test1.txt
+        @ 62fc20d (> repo-default-branch) create test1.txt
         "###);
     }
 
