@@ -8,12 +8,15 @@ mod core {
 
 mod command {
     mod test_amend;
+    mod test_bug_report;
     mod test_hide;
     mod test_init;
     mod test_move;
     mod test_navigation;
     mod test_restack;
+    mod test_reword;
     mod test_smartlog;
+    mod test_sync;
     mod test_undo;
     mod test_wrap;
 }
